"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateCredit = exports.calculatePricingInformation = exports.priceFormatter = void 0;
const geo_shipping_prices_1 = require("@uppfirst/geo-shipping-prices");
const decimal_js_1 = __importDefault(require("decimal.js"));
const utils_1 = require("./utils");
Object.defineProperty(exports, "priceFormatter", { enumerable: true, get: function () { return utils_1.priceFormatter; } });
const currencyCodeToSymbolMap = {
    usd: '$',
    eur: '€',
    gbp: '£',
    jpy: '¥',
    aed: 'AED',
};
const calculateShippingPrices = ({ product, stateCode, countryCode, traitCost, decimals, }) => {
    var _a, _b;
    const shipping = product.shipping;
    if (!shipping || !countryCode) {
        const zero = product.price instanceof decimal_js_1.default ? new decimal_js_1.default(0) : 0;
        return {
            taxCost: zero,
            taxPercent: zero,
            shippingCost: zero,
        };
    }
    const shippingCalculator = new geo_shipping_prices_1.Shipping(shipping);
    const prices = stateCode && countryCode === 'US'
        ? shippingCalculator.state(stateCode)
        : shippingCalculator.country(countryCode);
    const taxPercent = (0, utils_1.ensureDecimal)(((_a = prices.prices) === null || _a === void 0 ? void 0 : _a.tax) || 0);
    const shippingCost = (0, utils_1.ensureDecimal)(((_b = prices.prices) === null || _b === void 0 ? void 0 : _b.price) || 0);
    const price = (0, utils_1.ensureDecimal)(product.price);
    const calculatedShippingCost = (0, utils_1.multiplyByDecimals)(shippingCost, decimals);
    const taxableCost = price.plus(calculatedShippingCost).plus(traitCost);
    // Determine if we should return Decimal based on input type
    const shouldReturnDecimal = product.price instanceof decimal_js_1.default;
    const calculatedTaxCost = taxPercent
        .dividedBy(100)
        .times(taxableCost)
        .round();
    return {
        taxCost: shouldReturnDecimal
            ? calculatedTaxCost
            : calculatedTaxCost.toNumber(),
        taxPercent: shouldReturnDecimal ? taxPercent : taxPercent.toNumber(),
        shippingCost: calculatedShippingCost,
    };
};
/**
 * Calculate the credit gained for a product based on the quantity and the product's reward share.
 * @param product
 * @param quantity
 * @param opts
 */
const calculateCredit = (product, quantity, opts) => {
    const { returnSmallestUnit } = opts || {};
    const decimals = (0, utils_1.ensureDecimal)(product.currency.decimals);
    const startAmountPercent = (0, utils_1.ensureDecimal)(product.startAmountPercent).dividedBy(100);
    const rewardShare = (0, utils_1.ensureDecimal)(product.rewardShare).dividedBy(100);
    const claimPercent = new decimal_js_1.default(1).minus(startAmountPercent);
    const totalBase = (0, utils_1.ensureDecimal)(product.price).times(quantity);
    const baseReserve = totalBase.times(startAmountPercent);
    const baseClaim = totalBase.times(claimPercent);
    const gainedCreditReserve = rewardShare.times(baseReserve);
    const gainedCreditClaim = rewardShare.times(baseClaim);
    return {
        gainedCreditReserve: (0, utils_1.divideConvertBackSmallestUnit)(gainedCreditReserve, decimals, returnSmallestUnit),
        gainedCreditClaim: (0, utils_1.divideConvertBackSmallestUnit)(gainedCreditClaim, decimals, returnSmallestUnit),
        gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(gainedCreditReserve.plus(gainedCreditClaim), decimals, returnSmallestUnit),
    };
};
exports.calculateCredit = calculateCredit;
const calculatePricingInformation = ({ items, countryCode, stateCode, usedCreditReserve = 0, usedCreditClaim = 0, }, opts) => {
    const { returnSmallestUnit } = opts || {};
    if (items.length === 0) {
        throw new Error('No items provided for pricing calculation');
    }
    // Currency for all products of the seller is always the same
    const currency = items[0].product.currency;
    const decimals = (0, utils_1.ensureDecimal)(currency.decimals);
    // Calculate different prices for each asset item
    const itemCalculations = items.map(({ product, combination, quantity }) => {
        const startAmountPercent = (0, utils_1.ensureDecimal)(product.startAmountPercent).dividedBy(100);
        const claimPercent = new decimal_js_1.default(1).minus(startAmountPercent);
        const totalBase = (0, utils_1.ensureDecimal)(product.price).times(quantity);
        const traitCostSingle = (0, utils_1.ensureDecimal)((combination === null || combination === void 0 ? void 0 : combination.price) || 0);
        const traitCostAll = traitCostSingle.times(quantity);
        const baseReserve = totalBase.times(startAmountPercent);
        const baseClaim = totalBase.times(claimPercent);
        const { gainedCreditReserve, gainedCreditClaim, gainedCredit } = calculateCredit(product, quantity, { returnSmallestUnit: true });
        const { shippingCost, taxCost, taxPercent } = calculateShippingPrices({
            product,
            countryCode,
            stateCode,
            traitCost: traitCostSingle,
            decimals,
        });
        const shippingCostDecimal = (0, utils_1.ensureDecimal)(shippingCost).times(quantity);
        const taxCostDecimal = (0, utils_1.ensureDecimal)(taxCost).times(quantity);
        // Credit should be deducted in summary section
        const totalReserve = decimal_js_1.default.max(baseReserve
            .plus(traitCostAll)
            .plus(taxCostDecimal)
            .plus(shippingCostDecimal), new decimal_js_1.default(0));
        const total = decimal_js_1.default.max(totalReserve.plus(baseClaim), new decimal_js_1.default(0));
        return {
            product,
            combination,
            quantity,
            prices: {
                startAmountPercent: startAmountPercent.times(100).toNumber(),
                totalBase,
                baseReserve,
                baseClaim,
                traitCost: traitCostAll,
                gainedCreditReserveDecimal: (0, utils_1.ensureDecimal)(gainedCreditReserve),
                gainedCreditClaimDecimal: (0, utils_1.ensureDecimal)(gainedCreditClaim),
                gainedCreditDecimal: (0, utils_1.ensureDecimal)(gainedCredit),
                shippingCostDecimal,
                taxCostDecimal,
                taxPercentDecimal: (0, utils_1.ensureDecimal)(taxPercent),
                totalReserve,
                total,
            },
        };
    });
    const usedCreditReserveDecimal = (0, utils_1.ensureDecimal)(usedCreditReserve);
    const usedCreditClaimDecimal = (0, utils_1.ensureDecimal)(usedCreditClaim);
    const usedCreditDecimal = usedCreditReserveDecimal.plus(usedCreditClaimDecimal);
    return {
        items: itemCalculations.map(({ prices, quantity }) => ({
            startAmountPercent: prices.startAmountPercent,
            quantity,
            reserve: {
                base: (0, utils_1.divideConvertBackSmallestUnit)(prices.baseReserve, decimals, returnSmallestUnit),
                traitCost: (0, utils_1.divideConvertBackSmallestUnit)(prices.traitCost, decimals, returnSmallestUnit),
                gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(prices.gainedCreditReserveDecimal, decimals, returnSmallestUnit),
                taxPercent: (0, utils_1.convertBack)(prices.taxPercentDecimal, decimals),
                shippingCost: (0, utils_1.divideConvertBackSmallestUnit)(prices.shippingCostDecimal, decimals, returnSmallestUnit),
                taxCost: (0, utils_1.divideConvertBackSmallestUnit)(prices.taxCostDecimal, decimals, returnSmallestUnit),
                // Credit should be deducted in summary section
                total: (0, utils_1.divideConvertBackSmallestUnit)(prices.totalReserve, decimals, returnSmallestUnit),
            },
            claim: {
                base: (0, utils_1.divideConvertBackSmallestUnit)(prices.baseClaim, decimals, returnSmallestUnit),
                gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(prices.gainedCreditClaimDecimal, decimals, returnSmallestUnit),
                // Credit should be deducted in summary section
                total: (0, utils_1.divideConvertBackSmallestUnit)(prices.baseClaim, decimals, returnSmallestUnit),
            },
            aggregate: {
                base: (0, utils_1.divideConvertBackSmallestUnit)(prices.totalBase, decimals, returnSmallestUnit),
                gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(prices.gainedCreditDecimal, decimals, returnSmallestUnit),
                // Credit should be deducted in summary section
                total: (0, utils_1.divideConvertBackSmallestUnit)(prices.total, decimals, returnSmallestUnit),
            },
        })),
        summary: {
            currency: {
                code: currency.symbol.toLowerCase(),
                symbol: currencyCodeToSymbolMap[currency.symbol.toLocaleLowerCase()],
                decimals: (0, utils_1.convertBack)(decimals, 0),
            },
            usedCredit: (0, utils_1.divideConvertBackSmallestUnit)(usedCreditDecimal, decimals, returnSmallestUnit),
            gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(itemCalculations
                .map(({ prices }) => prices.gainedCreditDecimal)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
            baseReserve: (0, utils_1.divideConvertBackSmallestUnit)(itemCalculations
                .map(({ prices }) => prices.baseReserve)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
            traitCost: (0, utils_1.divideConvertBackSmallestUnit)(itemCalculations
                .map(({ prices }) => prices.traitCost)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
            taxCost: (0, utils_1.divideConvertBackSmallestUnit)(itemCalculations
                .map(({ prices }) => prices.taxCostDecimal)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
            shippingCost: (0, utils_1.divideConvertBackSmallestUnit)(itemCalculations
                .map(({ prices }) => prices.shippingCostDecimal)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
            totalReserve: (0, utils_1.divideConvertBackSmallestUnit)(decimal_js_1.default.max(itemCalculations
                .map(({ prices }) => prices.totalReserve)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0))
                .minus(usedCreditReserveDecimal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
            baseClaim: (0, utils_1.divideConvertBackSmallestUnit)(itemCalculations
                .map(({ prices }) => prices.baseClaim)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
            totalClaim: (0, utils_1.divideConvertBackSmallestUnit)(decimal_js_1.default.max(itemCalculations
                .map(({ prices }) => prices.baseClaim)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0))
                .minus(usedCreditReserveDecimal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
            baseTotal: (0, utils_1.divideConvertBackSmallestUnit)(itemCalculations
                .map(({ prices }) => prices.baseReserve)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0))
                .plus(itemCalculations
                .map(({ prices }) => prices.baseClaim)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0))), decimals, returnSmallestUnit),
            total: (0, utils_1.divideConvertBackSmallestUnit)(decimal_js_1.default.max(itemCalculations
                .map(({ prices }) => prices.total)
                .reduce((prev, curr) => prev.plus(curr), new decimal_js_1.default(0))
                .minus(usedCreditDecimal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
        },
    };
};
exports.calculatePricingInformation = calculatePricingInformation;
